@@ -4,7 +4,7 @@
  * Header file for the TclCurl extension to enable Tcl interpreters
  * to access libcurl.
  *
- * Copyright (c) 2001-2007 Andrs Garca Garca.
+ * Copyright (c) 2001-2009 Andres Garcia Garcia.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -42,7 +42,7 @@ extern "C" {
 #define TCL_STORAGE_CLASS DLLEXPORT
 #endif
 
-#define TclCurlVersion "7.17.1"
+#define TclCurlVersion "7.19.6"
 
 /*
  * This struct will contain the data of a transfer if the user wants
@@ -105,6 +105,7 @@ struct curlObjData {
     struct curl_slist      *http200aliases;
     char                   *command;
     int                     anyAuthFlag;
+    char                   *sshkeycallProc;
 };
 
 struct shcurlObjData {
@@ -114,7 +115,7 @@ struct shcurlObjData {
 
 #ifndef multi_h
 
-static CONST char *commandTable[] = {
+CONST static char *commandTable[] = {
     "setopt",
     "perform",
     "getinfo",
@@ -122,65 +123,74 @@ static CONST char *commandTable[] = {
     "configure",
     "duphandle",
     "reset",
+    "pause",
+    "resume",
     (char *) NULL
 };
 
-static CONST char *optionTable[] = {
-    "CURLOPT_URL",           "CURLOPT_FILE",           "CURLOPT_INFILE",
-    "CURLOPT_USERAGENT",     "CURLOPT_REFERER",        "CURLOPT_VERBOSE",
-    "CURLOPT_HEADER",        "CURLOPT_NOBODY",         "CURLOPT_PROXY",
-    "CURLOPT_PROXYPORT",     "CURLOPT_HTTPPROXYTUNNEL","CURLOPT_FAILONERROR",
-    "CURLOPT_TIMEOUT",       "CURLOPT_LOW_SPEED_LIMIT","CURLOPT_LOW_SPEED_TIME",
-    "CURLOPT_RESUME_FROM",   "CURLOPT_INFILESIZE",     "CURLOPT_UPLOAD",
-    "CURLOPT_FTPLISTONLY",   "CURLOPT_FTPAPPEND",      "CURLOPT_NETRC",
-    "CURLOPT_FOLLOWLOCATION","CURLOPT_TRANSFERTEXT",   "CURLOPT_PUT",
-    "CURLOPT_MUTE",          "CURLOPT_USERPWD",        "CURLOPT_PROXYUSERPWD",
-    "CURLOPT_RANGE",         "CURLOPT_ERRORBUFFER",    "CURLOPT_HTTPGET",
-    "CURLOPT_POST",          "CURLOPT_POSTFIELDS",     "CURLOPT_POSTFIELDSIZE",
-    "CURLOPT_FTPPORT",       "CURLOPT_COOKIE",         "CURLOPT_COOKIEFILE",
-    "CURLOPT_HTTPHEADER",    "CURLOPT_HTTPPOST",       "CURLOPT_SSLCERT",
-    "CURLOPT_SSLCERTPASSWD", "CURLOPT_SSLVERSION",     "CURLOPT_CRLF",
-    "CURLOPT_QUOTE",         "CURLOPT_POSTQUOTE",      "CURLOPT_WRITEHEADER",
-    "CURLOPT_TIMECONDITION", "CURLOPT_TIMEVALUE",      "CURLOPT_CUSTOMREQUEST",
-    "CURLOPT_STDERR",        "CURLOPT_INTERFACE",      "CURLOPT_KRB4LEVEL",
-    "CURLOPT_SSL_VERIFYPEER","CURLOPT_CAINFO",         "CURLOPT_FILETIME",
-    "CURLOPT_MAXREDIRS",     "CURLOPT_MAXCONNECTS",    "CURLOPT_CLOSEPOLICY",
-    "CURLOPT_RANDOM_FILE",   "CURLOPT_EGDSOCKET",      "CURLOPT_CONNECTTIMEOUT",
-    "CURLOPT_NOPROGRESS",    "CURLOPT_HEADERVAR",      "CURLOPT_BODYVAR",
+CONST static char *optionTable[] = {
+    "CURLOPT_URL",           "CURLOPT_FILE",            "CURLOPT_INFILE",
+    "CURLOPT_USERAGENT",     "CURLOPT_REFERER",         "CURLOPT_VERBOSE",
+    "CURLOPT_HEADER",        "CURLOPT_NOBODY",          "CURLOPT_PROXY",
+    "CURLOPT_PROXYPORT",     "CURLOPT_HTTPPROXYTUNNEL", "CURLOPT_FAILONERROR",
+    "CURLOPT_TIMEOUT",       "CURLOPT_LOW_SPEED_LIMIT", "CURLOPT_LOW_SPEED_TIME",
+    "CURLOPT_RESUME_FROM",   "CURLOPT_INFILESIZE",      "CURLOPT_UPLOAD",
+    "CURLOPT_FTPLISTONLY",   "CURLOPT_FTPAPPEND",       "CURLOPT_NETRC",
+    "CURLOPT_FOLLOWLOCATION","CURLOPT_TRANSFERTEXT",    "CURLOPT_PUT",
+    "CURLOPT_MUTE",          "CURLOPT_USERPWD",         "CURLOPT_PROXYUSERPWD",
+    "CURLOPT_RANGE",         "CURLOPT_ERRORBUFFER",     "CURLOPT_HTTPGET",
+    "CURLOPT_POST",          "CURLOPT_POSTFIELDS",      "CURLOPT_POSTFIELDSIZE",
+    "CURLOPT_FTPPORT",       "CURLOPT_COOKIE",          "CURLOPT_COOKIEFILE",
+    "CURLOPT_HTTPHEADER",    "CURLOPT_HTTPPOST",        "CURLOPT_SSLCERT",
+    "CURLOPT_SSLCERTPASSWD", "CURLOPT_SSLVERSION",      "CURLOPT_CRLF",
+    "CURLOPT_QUOTE",         "CURLOPT_POSTQUOTE",       "CURLOPT_WRITEHEADER",
+    "CURLOPT_TIMECONDITION", "CURLOPT_TIMEVALUE",       "CURLOPT_CUSTOMREQUEST",
+    "CURLOPT_STDERR",        "CURLOPT_INTERFACE",       "CURLOPT_KRB4LEVEL",
+    "CURLOPT_SSL_VERIFYPEER","CURLOPT_CAINFO",          "CURLOPT_FILETIME",
+    "CURLOPT_MAXREDIRS",     "CURLOPT_MAXCONNECTS",     "CURLOPT_CLOSEPOLICY",
+    "CURLOPT_RANDOM_FILE",   "CURLOPT_EGDSOCKET",       "CURLOPT_CONNECTTIMEOUT",
+    "CURLOPT_NOPROGRESS",    "CURLOPT_HEADERVAR",       "CURLOPT_BODYVAR",
     "CURLOPT_PROGRESSPROC","CURLOPT_CANCELTRANSVARNAME","CURLOPT_WRITEPROC",
     "CURLOPT_READPROC",      "CURLOPT_SSL_VERIFYHOST",  "CURLOPT_COOKIEJAR",
-    "CURLOPT_SSL_CIPHER_LIST","CURLOPT_HTTP_VERSION",  "CURLOPT_FTP_USE_EPSV",
-    "CURLOPT_SSLCERTTYPE",    "CURLOPT_SSLKEY",        "CURLOPT_SSLKEYTYPE",
-    "CURLOPT_SSLKEYPASSWD",   "CURLOPT_SSL_ENGINE",    "CURLOPT_SSL_ENGINEDEFAULT",
-    "CURLOPT_PREQUOTE",       "CURLOPT_DEBUGPROC",     "CURLOPT_DNS_CACHE_TIMEOUT",
+    "CURLOPT_SSL_CIPHER_LIST","CURLOPT_HTTP_VERSION",   "CURLOPT_FTP_USE_EPSV",
+    "CURLOPT_SSLCERTTYPE",    "CURLOPT_SSLKEY",         "CURLOPT_SSLKEYTYPE",
+    "CURLOPT_SSLKEYPASSWD",   "CURLOPT_SSL_ENGINE",     "CURLOPT_SSL_ENGINEDEFAULT",
+    "CURLOPT_PREQUOTE",       "CURLOPT_DEBUGPROC",      "CURLOPT_DNS_CACHE_TIMEOUT",
     "CURLOPT_DNS_USE_GLOBAL_CACHE", "CURLOPT_COOKIESESSION","CURLOPT_CAPATH",
-    "CURLOPT_BUFFERSIZE",     "CURLOPT_NOSIGNAL",      "CURLOPT_ENCODING",
-    "CURLOPT_PROXYTYPE",      "CURLOPT_HTTP200ALIASES","CURLOPT_UNRESTRICTED_AUTH",
-    "CURLOPT_FTP_USE_EPRT",   "CURLOPT_NOSUCHOPTION",  "CURLOPT_HTTPAUTH",
-    "CURLOPT_FTP_CREATE_MISSING_DIRS",                 "CURLOPT_PROXYAUTH",
-    "CURLOPT_FTP_RESPONSE_TIMEOUT",                    "CURLOPT_IPRESOLVE",
-    "CURLOPT_MAXFILESIZE",    "CURLOPT_NETRC_FILE",    "CURLOPT_FTP_SSL",
-    "CURLOPT_SHARE",          "CURLOPT_PORT",          "CURLOPT_TCP_NODELAY",
-    "CURLOPT_AUTOREFERER",    "CURLOPT_SOURCE_HOST",   "CURLOPT_SOURCE_USERPWD",
-    "CURLOPT_SOURCE_PATH",    "CURLOPT_SOURCE_PORT",   "CURLOPT_PASV_HOST",
+    "CURLOPT_BUFFERSIZE",     "CURLOPT_NOSIGNAL",       "CURLOPT_ENCODING",
+    "CURLOPT_PROXYTYPE",      "CURLOPT_HTTP200ALIASES", "CURLOPT_UNRESTRICTED_AUTH",
+    "CURLOPT_FTP_USE_EPRT",   "CURLOPT_NOSUCHOPTION",   "CURLOPT_HTTPAUTH",
+    "CURLOPT_FTP_CREATE_MISSING_DIRS",                  "CURLOPT_PROXYAUTH",
+    "CURLOPT_FTP_RESPONSE_TIMEOUT",                     "CURLOPT_IPRESOLVE",
+    "CURLOPT_MAXFILESIZE",    "CURLOPT_NETRC_FILE",     "CURLOPT_FTP_SSL",
+    "CURLOPT_SHARE",          "CURLOPT_PORT",           "CURLOPT_TCP_NODELAY",
+    "CURLOPT_AUTOREFERER",    "CURLOPT_SOURCE_HOST",    "CURLOPT_SOURCE_USERPWD",
+    "CURLOPT_SOURCE_PATH",    "CURLOPT_SOURCE_PORT",    "CURLOPT_PASV_HOST",
     "CURLOPT_SOURCE_PREQUOTE","CURLOPT_SOURCE_POSTQUOTE",
-    "CURLOPT_FTPSSLAUTH",     "CURLOPT_SOURCE_URL",    "CURLOPT_SOURCE_QUOTE",
+    "CURLOPT_FTPSSLAUTH",     "CURLOPT_SOURCE_URL",     "CURLOPT_SOURCE_QUOTE",
     "CURLOPT_FTP_ACCOUNT",    "CURLOPT_IGNORE_CONTENT_LENGTH",
     "CURLOPT_COOKIELIST",     "CURLOPT_FTP_SKIP_PASV_IP",
-    "CURLOPT_FTP_FILEMETHOD", "CURLOPT_LOCALPORT",     "CURLOPT_LOCALPORTRANGE",
-    "CURLOPT_MAX_SEND_SPEED_LARGE",                    "CURLOPT_MAX_RECV_SPEED_LARGE",
-    "CURLOPT_FTP_ALTERNATIVE_TO_USER",                 "CURLOPT_SSL_SESSIONID_CACHE",
-    "CURLOPT_SSH_AUTH_TYPES",                          "CURLOPT_SSH_PUBLIC_KEYFILE",
-    "CURLOPT_SSH_PRIVATE_KEYFILE",                     "CURLOPT_TIMEOUT_MS",
-    "CURLOPT_CONNECTTIMEOUT_MS",                       "CURLOPT_HTTP_CONTENT_DECODING",
-    "CURLOPT_HTTP_TRANSFER_DECODING",                  "CURLOPT_KRBLEVEL",
-    "CURLOPT_NEW_FILE_PERMS",                          "CURLOPT_NEW_DIRECTORY_PERMS",
-    "CURLOPT_KEYPASSWD",      "CURLOPT_APPEND",        "CURLOPT_DIRLISTONLY",
-    "CURLOPT_USE_SSL",        "CURLOPT_POST301",       "CURLOPT_SSH_HOST_PUBLIC_KEY_MD5",
-    (char *) NULL
+    "CURLOPT_FTP_FILEMETHOD", "CURLOPT_LOCALPORT",      "CURLOPT_LOCALPORTRANGE",
+    "CURLOPT_MAX_SEND_SPEED_LARGE",                     "CURLOPT_MAX_RECV_SPEED_LARGE",
+    "CURLOPT_FTP_ALTERNATIVE_TO_USER",                  "CURLOPT_SSL_SESSIONID_CACHE",
+    "CURLOPT_SSH_AUTH_TYPES",                           "CURLOPT_SSH_PUBLIC_KEYFILE",
+    "CURLOPT_SSH_PRIVATE_KEYFILE",                      "CURLOPT_TIMEOUT_MS",
+    "CURLOPT_CONNECTTIMEOUT_MS",                        "CURLOPT_HTTP_CONTENT_DECODING",
+    "CURLOPT_HTTP_TRANSFER_DECODING",                   "CURLOPT_KRBLEVEL",
+    "CURLOPT_NEW_FILE_PERMS",                           "CURLOPT_NEW_DIRECTORY_PERMS",
+    "CURLOPT_KEYPASSWD",      "CURLOPT_APPEND",         "CURLOPT_DIRLISTONLY",
+    "CURLOPT_USE_SSL",        "CURLOPT_POST301",        "CURLOPT_SSH_HOST_PUBLIC_KEY_MD5",
+    "CURLOPT_PROXY_TRANSFER_MODE",                      "CURLOPT_CRLFILE",
+    "CURLOPT_ISSUERCERT",     "CURLOPT_ADDRESS_SCOPE",  "CURLOPT_CERTINFO",
+    "CURLOPT_POSTREDIR",      "CURLOPT_USERNAME",       "CURLOPT_PASSWORD",
+    "CURLOPT_PROXYUSERNAME",  "CURLOPT_PROXYPASSWORD",  "CURLOPT_TFTP_BLKSIZE",
+    "CURLOPT_SOCKS5_GSSAPI_SERVICE",                    "CURLOPT_SOCKS5_GSSAPI_NEC",
+    "CURLOPT_PROTOCOLS",      "CURLOPT_REDIR_PROTOCOLS","CURLOPT_FTP_SSL_CC",
+    "CURLOPT_SSH_KNOWNHOSTS", "CURLOPT_SSH_KEYFUNCTION",
+    (char *)NULL
 };
 
-static CONST char *configTable[] = {
+CONST static char *configTable[] = {
     "-url",               "-file",               "-infile",
     "-useragent",         "-referer",            "-verbose",
     "-header",            "-nobody",             "-proxy",
@@ -230,16 +240,22 @@ static CONST char *configTable[] = {
     "-transferdecoding",  "-krblevel",           "-newfileperms",
     "-newdirectoryperms", "-keypasswd",          "-append",
     "-dirlistonly",       "-usessl",             "-post301",
-    "-sshhostpublickeymd5",
+    "-sshhostpublickeymd5",                      "-proxytransfermode",
+    "-crlfile",           "-issuercert",         "-addressscope",
+    "-certinfo",          "-postredir",          "-username",
+    "-password",          "-proxyuser",          "-proxypassword",
+    "-tftpblksize",       "-socks5gssapiservice","-socks5gssapinec",
+    "-protocols",         "-redirprotocols",     "-ftpsslcc",
+    "-sshknownhosts",     "-sshkeyproc",
     (char *) NULL
 };
 
-static CONST char    *timeCond[] = {
+CONST static char    *timeCond[] = {
     "ifmodsince", "ifunmodsince",
-    (char *) NULL
+    (char *)NULL
 };
 
-static CONST char    *getInfoTable[]={
+CONST static char    *getInfoTable[]={
     "effectiveurl",   "httpcode",       "responsecode",
     "filetime",       "totaltime",      "namelookuptime",
     "connecttime",    "pretransfertime","sizeupload",
@@ -250,78 +266,88 @@ static CONST char    *getInfoTable[]={
     "redirecttime",   "redirectcount",  "httpauthavail",
     "proxyauthavail", "oserrno",        "numconnects",
     "sslengines",     "httpconnectcode","cookielist",
-    "ftpentrypath",
-    (char *) NULL
+    "ftpentrypath",   "redirecturl",    "primaryip",
+    "appconnecttime", "certinfo",       "conditionunmet",
+    (char *)NULL
 };
 
-static CONST char   *curlFormTable[]={
+CONST static char   *curlFormTable[]={
     "name",  "contents", "file", "contenttype", "contentheader", "filename",
     "bufferName", "buffer", "filecontent", (char *)NULL
 };
 
-static CONST char   *httpVersionTable[] = {
+CONST static char   *httpVersionTable[] = {
     "none", "1.0", "1.1", (char *)NULL
 };
 
-static CONST char *netrcTable[] = {
+CONST static char *netrcTable[] = {
     "optional", "ignored", "required", (char *)NULL
 };
 
-static CONST char *encodingTable[] = {
+CONST static char *encodingTable[] = {
     "identity", "deflated", "all", (char *)NULL
 };
 
-static CONST char *versionInfoTable[] = {
+CONST static char *versionInfoTable[] = {
     "-version",    "-versionnum",    "-host",         "-features",
     "-sslversion", "-sslversionnum", "-libzversion",
     "-protocols",  (char *)NULL
 };
 
-static CONST char *proxyTypeTable[] = {
-    "http", "socks5", "socks4", (char *)NULL
+CONST static char *proxyTypeTable[] = {
+    "http", "socks4", "socks4a", "socks5", "socks5hostname", (char *)NULL
 };
 
-static CONST char *httpAuthMethods[] = {
-    "basic", "digest", "gssnegotiate", "ntlm", "any", "anysafe", (char *)NULL
+CONST static char *httpAuthMethods[] = {
+    "basic", "digest", "digestie", "gssnegotiate", "ntlm", "any", "anysafe", (char *)NULL
 };
 
-static CONST char *ipresolve[] = {
+CONST static char *ipresolve[] = {
     "whatever", "v4", "v6", (char *)NULL
 };
 
-static CONST char *ftpssl[] = {
+CONST static char *ftpssl[] = {
     "nope", "try", "control", "all", (char *)NULL
 };
 
-static CONST char *shareCmd[] = {
+CONST static char *shareCmd[] = {
     "share", "unshare", "cleanup", (char *)NULL
 };
 
-static CONST char *lockData[] = {
+CONST static char *lockData[] = {
     "cookies", "dns", (char *)NULL
 };
 
-static CONST char *ftpsslauth[] = {
+CONST static char *ftpsslauth[] = {
     "default", "ssl", "tls", (char *)NULL
 };
 
-static CONST char *ftpsslccc[] = {
+CONST static char *ftpsslccc[] = {
     "none", "passive", "active", (char *)NULL
 };
 
-static CONST char *sslversion[] = {
+CONST static char *sslversion[] = {
     "default", "tlsv1", "sslv2", "sslv3", (char *)NULL
 };
 
-static CONST char *ftpfilemethod[] = {
+CONST static char *ftpfilemethod[] = {
     "default", "multicwd", "nocwd", "singlecwd", (char *)NULL
 };
 
-static CONST char *sshauthtypes[] = {
+CONST static char *sshauthtypes[] = {
     "publickey", "password", "host", "keyboard", "any", (char *)NULL
 };
 
-static curlioerr curlioctl(CURL *handle, curliocmd cmd, void *userp);
+CONST static char *postredir[] = {
+    "301", "302", "all", (char *)NULL
+};
+
+CONST static char *protocolNames[] = {
+    "http", "https", "ftp", "ftps", "scp", "sftp", "telnet", "ldap",
+    "ldaps","dict",  "file","tftp", "all", (char*)NULL
+};
+
+static curlioerr curlseek(void *instream, curl_off_t offset, int origin);
 
 int Tclcurl_MultiInit (Tcl_Interp *interp);
 
@@ -369,7 +395,7 @@ CURLcode curlGetInfo(Tcl_Interp *interp,CURL *curlHandle,int tableIndex);
 
 void curlFreeSpace(struct curlObjData *curlData);
 
-void curlErrorSetOpt(Tcl_Interp *interp,CONST char **configTable, int option, CONST char *parPtr);
+void curlErrorSetOpt(Tcl_Interp *interp,CONST char **configTable, int option,CONST char *parPtr);
 
 size_t curlHeaderReader(void *ptr,size_t size,size_t nmemb,FILE *stream);
 
@@ -381,6 +407,18 @@ int curlProgressCallback(void *clientp,double dltotal,double dlnow,
 size_t curlWriteProcInvoke(void *ptr,size_t size,size_t nmemb,FILE *curlDataPtr);
 size_t curlReadProcInvoke(void *ptr,size_t size,size_t nmemb,FILE *curlDataPtr);
 
+
+/* Puts a ssh key into a Tcl object */
+Tcl_Obj *curlsshkeyextract(Tcl_Interp *interp,const struct curl_khkey *key);
+
+/* Function that will be invoked by libcurl to see what the user wants to
+   do about the new ssh host */
+size_t curlsshkeycallback(CURL *easy,                        /* easy handle */
+                          const struct curl_khkey *knownkey, /* known */
+                          const struct curl_khkey *foundkey, /* found */
+                          enum curl_khmatch,                 /* libcurl's view on the keys */
+                          void *curlData);
+
 int curlDebugProcInvoke(CURL *curlHandle, curl_infotype infoType,
         unsigned char * dataPtr, size_t size, void  *curlData);
 
