@@ -6,6 +6,8 @@ $curlHandle configure -url "127.0.0.1" -filetime 1  -verbose 1             \
 $curlHandle perform
 
 puts "Url: [$curlHandle getinfo effectiveurl]"
+puts "Primary IP: [$curlHandle getinfo primaryip]"
+puts "Redirect url: [$curlHandle getinfo redirecturl]"
 puts "Http-code: [$curlHandle getinfo responsecode]"
 puts "Proxy response code: [$curlHandle getinfo httpconnectcode]"
 set fileTime [$curlHandle getinfo filetime]
@@ -15,6 +17,7 @@ puts "Name lookup time: [$curlHandle getinfo namelookuptime]"
 puts "Name connect time: [$curlHandle getinfo connecttime]"
 puts "Name pretransfer time: [$curlHandle getinfo pretransfertime]"
 puts "Name start transfer time: [$curlHandle getinfo starttransfertime]"
+puts "Name app connect time: [$curlHandle getinfo appconnecttime]"
 puts "Name size upload: [$curlHandle getinfo sizeupload]"
 puts "Name size download: [$curlHandle getinfo sizedownload]"
 puts "Name speed download: [$curlHandle getinfo speeddownload]"
@@ -33,6 +36,12 @@ puts "Authentication methods at the proxy: [$curlHandle getinfo proxyauthavail]"
 puts "Operating System error number: [$curlHandle getinfo oserrno]"
 puts "Number of successful connects: [$curlHandle getinfo numconnects]"
 puts "Known cookies: [$curlHandle getinfo cookielist]"
+set certList [$curlHandle getinfo certinfo]
+set certNum  [lindex $certList 0]
+puts "N de certificados: $certNum"
+for {set i 1} {$i<=$certNum} {incr i} {
+    puts [lindex $certList $i]
+}
 
 $curlHandle cleanup
 
